/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#include "Constant.h"

namespace HARK
{
    DECLARE_NODE(Constant);
    /*Node
     *
     * @name Constant
     * @category General
     * @description Defines a constant in terms of type and value. The different types are: int, float, bool, string, objects and subnet_param.
     *
     * @output_name VALUE
     * @output_description The value (parameter)
     *
     * @parameter_name VALUE
     * @parameter_description The value
     *
     END*/

    Constant::Constant(const std::string &nodeName, const ParameterSet &params) :
	Node(nodeName, params)
    {
	this->_outputID = addOutput("VALUE");
    }

    void
    Constant::initialize()
    {
	this->_value = parameters.get("VALUE");
	this->Node::initialize();
    }

    ObjectRef
    Constant::getOutput(int output_id, int count)
    {
	if (output_id == this->_outputID) {
	    return this->_value;
	} else {
	    throw new NodeException (this, "Constant: Unknown output id", __FILE__, __LINE__);
	}
    }
}

/* end of file */
